package com.jjt.calc.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.enums.BusinessType;
import com.jjt.calc.domain.TwinFormulaInfo;
import com.jjt.calc.service.ITwinFormulaInfoService;
import com.jjt.common.utils.poi.ExcelUtil;
import com.jjt.common.core.page.TableDataInfo;

/**
 * 设备配方信息Controller
 *
 * @author wukai
 * @date 2025-01-18
 */
@Api(tags="设备配方信息")
@RestController
@RequestMapping("/calc/formulaInfo")
public class TwinFormulaInfoController extends BaseController{
    @Resource
    private ITwinFormulaInfoService twinFormulaInfoService;

    /**
     * 查询设备配方信息列表
     */
    @ApiOperation("查询设备配方信息列表")
    @PreAuthorize("@ss.hasPermi('calc:formulaInfo:list')")
    @GetMapping("/list")
    public TableDataInfo list(TwinFormulaInfo twinFormulaInfo)
    {
        startPage();
        List<TwinFormulaInfo> list = twinFormulaInfoService.selectTwinFormulaInfoList(twinFormulaInfo);
        return getDataTable(list);
    }

    /**
     * 导出设备配方信息列表
     */
    @ApiOperation("导出设备配方信息列表")
    @PreAuthorize("@ss.hasPermi('calc:formulaInfo:export')")
    @Log(title = "设备配方信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TwinFormulaInfo twinFormulaInfo)
    {
        List<TwinFormulaInfo> list = twinFormulaInfoService.selectTwinFormulaInfoList(twinFormulaInfo);
        ExcelUtil<TwinFormulaInfo> util = new ExcelUtil<TwinFormulaInfo>(TwinFormulaInfo.class);
        util.exportExcel(response, list, "设备配方信息数据");
    }

    /**
     * 获取设备配方信息详细信息
     */
    @ApiOperation("获取设备配方信息详细信息")
    @PreAuthorize("@ss.hasPermi('calc:formulaInfo:query')")
    @GetMapping(value = "/{specInfoId}")
    public AjaxResult getInfo(@PathVariable("specInfoId") Long specInfoId)
    {
        return success(twinFormulaInfoService.selectTwinFormulaInfoBySpecInfoId(specInfoId));
    }

    /**
     * 新增设备配方信息
     */
    @ApiOperation("新增设备配方信息")
    @PreAuthorize("@ss.hasPermi('calc:formulaInfo:add')")
    @Log(title = "设备配方信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TwinFormulaInfo twinFormulaInfo)
    {
        return toAjax(twinFormulaInfoService.insertTwinFormulaInfo(twinFormulaInfo));
    }

    /**
     * 修改设备配方信息
     */
    @ApiOperation("修改设备配方信息")
    @PreAuthorize("@ss.hasPermi('calc:formulaInfo:edit')")
    @Log(title = "设备配方信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TwinFormulaInfo twinFormulaInfo)
    {
        return toAjax(twinFormulaInfoService.updateTwinFormulaInfo(twinFormulaInfo));
    }

    /**
     * 删除设备配方信息
     */
    @ApiOperation("删除设备配方信息")
    @PreAuthorize("@ss.hasPermi('calc:formulaInfo:remove')")
    @Log(title = "设备配方信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{specInfoIds}")
    public AjaxResult remove(@PathVariable Long[] specInfoIds)
    {
        return toAjax(twinFormulaInfoService.deleteTwinFormulaInfoBySpecInfoIds(specInfoIds));
    }
}
