package com.jjt.calc.mapper;

import java.util.Date;
import java.util.List;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.jjt.calc.domain.TwinCalcDay;
import org.apache.ibatis.annotations.Param;

/**
 * 日统计数据Mapper接口
 *
 * @author wukai
 * @date 2025-01-18
 */
public interface TwinCalcDayMapper extends BaseMapper<TwinCalcDay>
{
    /**
     * 查询日统计数据
     *
     * @param id 日统计数据主键
     * @return 日统计数据
     */
    public TwinCalcDay selectTwinCalcDayById(Long id);

    /**
     * 查询日统计数据列表
     *
     * @param twinCalcDay 日统计数据
     * @return 日统计数据集合
     */
    public List<TwinCalcDay> selectTwinCalcDayList(TwinCalcDay twinCalcDay);

    /**
     * 新增日统计数据
     *
     * @param twinCalcDay 日统计数据
     * @return 结果
     */
    public int insertTwinCalcDay(TwinCalcDay twinCalcDay);

    /**
     * 修改日统计数据
     *
     * @param twinCalcDay 日统计数据
     * @return 结果
     */
    public int updateTwinCalcDay(TwinCalcDay twinCalcDay);

    /**
     * 删除日统计数据
     *
     * @param id 日统计数据主键
     * @return 结果
     */
    public int deleteTwinCalcDayById(Long id);

    /**
     * 批量删除日统计数据
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTwinCalcDayByIds(Long[] ids);
    /**
     * 查询指定日期及之后的数据
     *
     * @param sd 开始日期
     * @param ed 结束日期
     * @return 列表
     */
    List<TwinCalcDay> selectTwinCalcDayListByTime(@Param("sd") Date sd, @Param("ed") Date ed);


    /**
     * 按日期删除数据
     *
     * @param date
     */
    void delete4date(Date date);
    /**
     * 按月度查询
     *
     * @param month 月份
     * @return 结果
     */
    List<TwinCalcDay> selectTwinCalcDayListByMonth(String month);
}
