package com.jjt.inventory.domain;

import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.jjt.common.annotation.Excel;
import com.jjt.common.core.domain.BaseEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Date;

/**
 * 盘头仓库小时数据对象 TWIN_CALC_HOUR_PT
 *
 * @author wukai
 * @date 2025-06-10
 */
@ApiModel(value = "TwinCalcHourPt", description = "盘头仓库小时数据")
@NoArgsConstructor
@Data
public class TwinCalcHourPt extends BaseEntity {
    private static final long serialVersionUID = 1L;

    /**
     * ID
     */
    @ApiModelProperty("ID")
    @TableId
    private Long id;

    /**
     * 日期
     */
    @ApiModelProperty("日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "日期", width = 30, dateFormat = "yyyy-MM-dd")
    private Date dataDate;

    /**
     * 小时;0-23
     */
    @ApiModelProperty("小时;0-23")
    @Excel(name = "小时;0-23")
    private Integer hour;

    /**
     * 数据名称
     */
    @ApiModelProperty("数据名称")
    @Excel(name = "数据名称")
    private String name;

    /**
     * SKU
     */
    @ApiModelProperty("SKU")
    @Excel(name = "SKU")
    private String sku;

    /**
     * 入库数量
     */
    @ApiModelProperty("入库数量")
    @Excel(name = "入库数量")
    private Integer inNum;

    /**
     * 出库数量
     */
    @ApiModelProperty("出库数量")
    @Excel(name = "出库数量")
    private Integer outNum;

    /**
     * 当前库存
     */
    @ApiModelProperty("当前库存")
    @Excel(name = "当前库存")
    private Integer num;

    /**
     * 创建人
     */
    @ApiModelProperty("创建人")
    @Excel(name = "创建人")
    private String createdBy;

    /**
     * 创建时间
     */
    @ApiModelProperty("创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "创建时间", width = 30, dateFormat = "yyyy-MM-dd")
    private Date createdTime;

    /**
     * 更新人
     */
    @ApiModelProperty("更新人")
    @Excel(name = "更新人")
    private String updatedBy;

    /**
     * 更新时间
     */
    @ApiModelProperty("更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "更新时间", width = 30, dateFormat = "yyyy-MM-dd")
    private Date updatedTime;

    public TwinCalcHourPt(boolean flag) {
        inNum = 0;
        outNum = 0;
        num = 0;
    }

    /**
     * 增加入库数量
     *
     * @param value 要增加的值
     */
    public void addInNum(Integer value) {
        this.inNum += value;
    }

    /**
     * 增加出库数量
     *
     * @param value 要增加的值
     */
    public void addOutNum(Integer value) {
        this.outNum += value;
    }

    /**
     * 增加当前库存
     *
     * @param value 要增加的值
     */
    public void addNum(Integer value) {
        this.num += value;
    }
}
