package com.jjt.inventory.service;

import com.jjt.inventory.domain.TwinCalcHourBp;

import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 白坯仓库小时数据Service接口
 *
 * @author wukai
 * @date 2025-06-10
 */
public interface ITwinCalcHourBpService {
    /**
     * 查询白坯仓库小时数据
     *
     * @param id 白坯仓库小时数据主键
     * @return 白坯仓库小时数据
     */
    public TwinCalcHourBp selectTwinCalcHourBpById(Long id);

    /**
     * 查询白坯仓库小时数据列表
     *
     * @param twinCalcHourBp 白坯仓库小时数据
     * @return 白坯仓库小时数据集合
     */
    public List<TwinCalcHourBp> selectTwinCalcHourBpList(TwinCalcHourBp twinCalcHourBp);

    /**
     * 新增白坯仓库小时数据
     *
     * @param twinCalcHourBp 白坯仓库小时数据
     * @return 结果
     */
    public int insertTwinCalcHourBp(TwinCalcHourBp twinCalcHourBp);

    /**
     * 修改白坯仓库小时数据
     *
     * @param twinCalcHourBp 白坯仓库小时数据
     * @return 结果
     */
    public int updateTwinCalcHourBp(TwinCalcHourBp twinCalcHourBp);

    /**
     * 批量删除白坯仓库小时数据
     *
     * @param ids 需要删除的白坯仓库小时数据主键集合
     * @return 结果
     */
    public int deleteTwinCalcHourBpByIds(Long[] ids);

    /**
     * 删除白坯仓库小时数据信息
     *
     * @param id 白坯仓库小时数据主键
     * @return 结果
     */
    public int deleteTwinCalcHourBpById(Long id);

    /**
     * 删除白坯仓库小时数据信息
     *
     * @param date 日期
     * @param hour 小时
     */
    void delete(LocalDate date, int hour);

    /**
     * 按天查询数据
     *
     * @param date 日期
     * @return 结果
     */
    List<TwinCalcHourBp> selectTwinCalcHourPtByDay(LocalDate date);

    /**
     * 按照天统计数据查询
     *
     * @param date 日期
     * @param hours 需要查询的小时
     * @return 以小时分组的结果
     */
    Map<Integer, List<TwinCalcHourBp>> selectTwinCalcHourBpByHours(Date date, int[] hours);
}
