package com.jjt.inventory.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jjt.common.utils.DateUtils;
import com.jjt.inventory.domain.TwinCalcHourBp;
import com.jjt.inventory.mapper.TwinCalcHourBpMapper;
import com.jjt.inventory.service.ITwinCalcHourBpService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 白坯仓库小时数据Service业务层处理
 *
 * @author wukai
 * @date 2025-06-10
 */
@Service
public class TwinCalcHourBpServiceImpl implements ITwinCalcHourBpService {
    @Resource
    private TwinCalcHourBpMapper twinCalcHourBpMapper;

    /**
     * 查询白坯仓库小时数据
     *
     * @param id 白坯仓库小时数据主键
     * @return 白坯仓库小时数据
     */
    @Override
    public TwinCalcHourBp selectTwinCalcHourBpById(Long id) {
        return twinCalcHourBpMapper.selectTwinCalcHourBpById(id);
    }

    /**
     * 查询白坯仓库小时数据列表
     *
     * @param twinCalcHourBp 白坯仓库小时数据
     * @return 白坯仓库小时数据
     */
    @Override
    public List<TwinCalcHourBp> selectTwinCalcHourBpList(TwinCalcHourBp twinCalcHourBp) {
        return twinCalcHourBpMapper.selectTwinCalcHourBpList(twinCalcHourBp);
    }

    /**
     * 新增白坯仓库小时数据
     *
     * @param twinCalcHourBp 白坯仓库小时数据
     * @return 结果
     */
    @Override
    public int insertTwinCalcHourBp(TwinCalcHourBp twinCalcHourBp) {
        return twinCalcHourBpMapper.insertTwinCalcHourBp(twinCalcHourBp);
    }

    /**
     * 修改白坯仓库小时数据
     *
     * @param twinCalcHourBp 白坯仓库小时数据
     * @return 结果
     */
    @Override
    public int updateTwinCalcHourBp(TwinCalcHourBp twinCalcHourBp) {
        return twinCalcHourBpMapper.updateTwinCalcHourBp(twinCalcHourBp);
    }

    /**
     * 批量删除白坯仓库小时数据
     *
     * @param ids 需要删除的白坯仓库小时数据主键
     * @return 结果
     */
    @Override
    public int deleteTwinCalcHourBpByIds(Long[] ids) {
        return twinCalcHourBpMapper.deleteTwinCalcHourBpByIds(ids);
    }

    /**
     * 删除白坯仓库小时数据信息
     *
     * @param id 白坯仓库小时数据主键
     * @return 结果
     */
    @Override
    public int deleteTwinCalcHourBpById(Long id) {
        return twinCalcHourBpMapper.deleteTwinCalcHourBpById(id);
    }

    /**
     * 删除白坯仓库小时数据信息
     *
     * @param date 日期
     * @param hour 小时
     */
    @Override
    public void delete(LocalDate date, int hour) {
        QueryWrapper<TwinCalcHourBp> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("data_date", date);
        queryWrapper.eq("hour", hour);
        twinCalcHourBpMapper.delete(queryWrapper);
    }

    /**
     * 查询白坯仓库小时数据
     *
     * @param date 日期
     * @return 白坯仓库小时数据
     */
    @Override
    public List<TwinCalcHourBp> selectTwinCalcHourPtByDay(LocalDate date) {
        TwinCalcHourBp query = new TwinCalcHourBp();
        query.setDataDate(DateUtils.toDate(date));
        return selectTwinCalcHourBpList(query);
    }

    /**
     * 按照天统计数据查询
     *
     * @param date 日期
     * @param hours 需要查询的小时
     * @return 以小时分组的结果
     */
    @Override
    public Map<Integer, List<TwinCalcHourBp>> selectTwinCalcHourBpByHours(Date date, int[] hours) {
        List<TwinCalcHourBp> list = twinCalcHourBpMapper.selectTwinCalcHourBpByHours(date, hours);
        return list.stream().collect(Collectors.groupingBy(TwinCalcHourBp::getHour));
    }
}
